// Rot13Cipher - the rot13 trivial encryption method
//
// Copyright (C) 1996 by Jef Poskanzer <jef@acme.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit the ACME Labs Java page for up-to-date versions of this and other
// fine Java utilities: http://www.acme.com/java/

package Acme.Crypto;

import java.io.*;

/// The rot13 trivial encryption method.
// <P>
// This is mainly for testing stream ciphers.
// <P>
// <A HREF="/resources/classes/Acme/Crypto/Rot13Cipher.java">Fetch the software.</A><BR>
// <A HREF="/resources/classes/Acme.tar.gz">Fetch the entire Acme package.</A>

public class Rot13Cipher extends StreamCipher
    {

    /// Constructor.
    public Rot13Cipher()
	{
	super( 0 );
	}

    /// Rot13 doesn't have a key, but we must define this routine anyway
    // to avoid a compilation error.
    public void setKey( byte[] key )
	{
	throw new InternalError( "rot13 does not need a key" );
	}

    /// Encrypt a byte.
    public byte encrypt( byte clearText )
	{
	return (byte) rot13tab[clearText & 0xff];
	}

    /// Decrypt a byte.
    public byte decrypt( byte cipherText )
	{
	return (byte) rot13tab[cipherText & 0xff];
	}

    /// Encrypt some bytes.
    public void encrypt( byte[] clearText, int clearOff, byte[] cipherText, int
cipherOff, int len )
        {
        for ( int i = 0; i < len; ++i )
            cipherText[cipherOff + i] =
                (byte) rot13tab[clearText[clearOff + i] & 0xff];
        }

    /// Decrypt some bytes.
    public void decrypt( byte[] cipherText, int cipherOff, byte[] clearText, int
 clearOff, int len )
        {
        for ( int i = 0; i < len; ++i )
            clearText[clearOff + i] =
                (byte) rot13tab[cipherText[cipherOff + i] & 0xff];
        }

    private static int[] rot13tab = {
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,	// NUL - BEL
	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,	// BS - SI
	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,	// DLE - ETB
	0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,	// CAN - US
	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,	// SP - '
	0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,	// ( - /
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,	// 0 - 7
	0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,	// 8 - ?

	0x40, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,	// @ - G
	0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x41, 0x42,	// H - O
	0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a,	// P - W
	0x4b, 0x4c, 0x4d, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,	// X - _

	0x60, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74,	// ` - g
	0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x61, 0x62,	// h - o
	0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a,	// p - w
	0x6b, 0x6c, 0x6d, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,	// x - DEL

	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,	// NUL - BEL
	0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,	// BS - SI
	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97,	// DLE - ETB
	0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,	// CAN - US
	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,	// SP - '
	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,	// ( - /
	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7,	// 0 - 7
	0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,	// 8 - ?

	0xc0, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3, 0xd4,	// @ - G
	0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xc1, 0xc2,	// H - O
	0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca,	// P - W
	0xcb, 0xcc, 0xcd, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,	// X - _

	0xe0, 0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4,	// ` - g
	0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xe1, 0xe2,	// h - o
	0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,	// p - w
	0xeb, 0xec, 0xed, 0xfb, 0xfc, 0xfd, 0xfe, 0xff,	// x - DEL
	};

    }
