/*
 *	EasyScout Boy Scouts of America tracking software.
 *	http://www.easyscout.org/
 *
 *	Copyright (C) 2000 Conan C. Albrecht
 *	conan@warp.cx
 *
 *	This program is released under the GNU license so tech-savvy
 *	Boy Scouts and others can enhance the program for the use
 *	of all.  Please e-mail changes to my address above so I can
 *	reintegrate into the main program at http://www.easyscout.org/.
 *	This way, all users mutually benefit one another in the spirit of
 *	scouting.
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 * 	Any and all changes to the program code must be documented with
 *  date, author name and e-mail, and a short description.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package cx.warp.scout;

import java.util.*;

/** The interface to a person */
public interface Person extends Identifiable {

	/** Explicitly saves this person's information */
	public void save() throws DataException;

	/** Sets the person's first name */
	public void setFirstName(String fname) throws DataException;
	
	/** Returns the person's first name */
	public String getFirstName() throws DataException;
	
	/** Sets the person's middle name */
	public void setMiddleName(String mname) throws DataException;

	/** Returns the person's middle name */
	public String getMiddleName() throws DataException;
	
	/** Sets the person's last name */
	public void setLastName(String lname) throws DataException;

	/** Returns the person's last name */
	public String getLastName()throws DataException;
	
	/** Sets the person's birthdate */
	public void setBirthDate(Date birth)throws DataException;
	
	/** Returns the person's birthdate */
	public Date   getBirthDate()throws DataException;
	
	/** Sets the person's address */
	public void setAddress(String address)throws DataException;
	
	/** Returns the person's address */
	public String getAddress()throws DataException;
	
	/** Sets the person's e-mail */
	public void setEmail(String email)throws DataException;
	
	/** Returns the person's e-mail address */
	public String getEmail()throws DataException;
	
	/** Sets the person's phone number */
	public void setPhone(String phone)throws DataException;
	
	/** Returns the person's phone number */
	public String getPhone()throws DataException;

	/** Sets the notes about the person */
	public void setNotes(String notes)throws DataException;
	
	/** Returns notes about the person */
	public String getNotes()throws DataException;
	
	/** Returns the person's name, in the style: first middle last */
	public String getName()throws DataException;
	
	/** Returns the person's name, in the style: last, first middle */
	public String getFullName()throws DataException;
	
	/** Returns the person's name in the style: first initial. last name */
	public String getSmallName() throws DataException;
	
	/** Returns the person's current position */
	public String getPosition()throws DataException;

	/** Returns the person's age as of today */
	public String getAge()throws DataException;

	/** Sets the password for the person */
	public void setPassword(String pass) throws DataException;
	
	/** Returns the current password */
	public String getPassword() throws DataException;
	
	/** Verifies a password for this person */
	public boolean verifyPassword(String pass) throws DataException;
	


}//Person interface
