/*
 *	EasyScout Boy Scouts of America tracking software.
 *	http://www.easyscout.org/
 *
 *	Copyright (C) 2000 Conan C. Albrecht
 *	conan@warp.cx
 *
 *	This program is released under the GNU license so tech-savvy
 *	Boy Scouts and others can enhance the program for the use
 *	of all.  Please e-mail changes to my address above so I can
 *	reintegrate into the main program at http://www.easyscout.org/.
 *	This way, all users mutually benefit one another in the spirit of
 *	scouting.
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 * 	Any and all changes to the program code must be documented with
 *  date, author name and e-mail, and a short description.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package cx.warp.scout;

import java.util.*;

/** A scout rank, such as Tenderfoot, Scout, etc. */
public interface Rank {

	/** Returns the name of this rank */
	public String getName() throws DataException ;

	/** Returns whether all requirements are passed off for this rank */
	public boolean isCompleted() throws DataException;
	
	/** Returns the number of requirements completed for this rank */
	public int getCompletedCount() throws DataException;
	
	/** Returns the date the last item was passed off for this rank (which should always be the board of review),
			or null if the rank is not yet completed. */
	public Date getCompletedDate() throws DataException;
	
	/** Returns the date this rank was actually awarded to the scout, or null if the rank has not been awarded yet */
	public Date getAwarded() throws DataException;
	
	/** Sets the date this rank was actually awarded */
	public void setAwarded(Date awarded) throws DataException;
	
	/** Returns the number of requirements */
	public int getRequirementCount() throws DataException;
	
	/** Gets the given requirement */
	public Requirement getRequirement(int index) throws DataException;
	
	/** Returns all of the requirements for this rank */
	public Enumeration getRequirements() throws DataException;

	/** Returns the number of required merit badges for this rank */
	public int getRequiredCount() throws DataException;
	
	/** Returns the total number of merit badges for this rank */
	public int getMeritBadgeCount() throws DataException;

}//Rank
