/*
 *	EasyScout Boy Scouts of America tracking software.
 *	http://www.easyscout.org/
 *
 *	Copyright (C) 2000 Conan C. Albrecht
 *	conan@warp.cx
 *
 *	This program is released under the GNU license so tech-savvy
 *	Boy Scouts and others can enhance the program for the use
 *	of all.  Please e-mail changes to my address above so I can
 *	reintegrate into the main program at http://www.easyscout.org/.
 *	This way, all users mutually benefit one another in the spirit of
 *	scouting.
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 * 	Any and all changes to the program code must be documented with
 *  date, author name and e-mail, and a short description.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package cx.warp.scout.file;

import cx.warp.scout.*;
import java.util.*;
import java.io.*;

/** A requirement for a rank */
public class FileRequirement implements Identifiable, Requirement, Serializable {
	
	public static final long serialVersionUID = -7698546303468273617L;
	
	String id = Utils.getGUID();
	String index = null;
	String text = null;
	Date passed = null;

	/** Constructor */
	public FileRequirement(String index, String text) {
		this.index = index;
		this.text = text;
	}//constructor
	
	/** Returns the unique id for this object */
	public String getID() {
		return id;
	}//getID
	
	/** Sets the index number for this requirement */
	public void setIndex(String index) throws DataException {
		this.index = index;
	}//setIndex
	
	/** Returns the index number for this requirement */
	public String getIndex() throws DataException {
		return index;
	}//getIndex
	
	/** Sets the name of this requirement */
	public void setName(String name) throws DataException {
		this.text = name;
	}//setName
	
	/** Returns the name of this requirement */
	public String getName() throws DataException {
		return text;
	}//getText

	/** Returns the text (index + name) of this requirement */
	public String getText() throws DataException {
		return index + ". " + text;
	}//getText
	
	/** Sets the date that this requirement was passed off */
	public void setPassed(Date passed) throws DataException {
		this.passed = passed;
	}//setPassed
	
	/** Returns the dat that this requirement was passed off, or null if the requirement is not passed off yet. */
	public Date getPassed() throws DataException {
		return passed;
	}//getPassed
	
	/** Prints debug information */
	public String toString() {
		return index + ". " + text + (passed == null ? "" : passed.toString());
	}//toString

	/** Writes my information to an XML buffer */
	public void writeXML(PrintWriter out) throws DataException {
		out.println("      <Requirement id=\"" + id + "\">");
		out.println("        <Index>" + Utils.xml(index) + "</Index>");
		out.println("        <Name>" + Utils.xml(text) + "</Name>");
		out.println("        <Passed>" + Utils.xml(passed) + "</Passed>");
		out.println("      </Requirement>");
	}//writeXML

}//FileRequirement
